;Sign-extend
;https://www.nesdev.org/wiki/Synthetic_instructions

;When increasing the number of bits in a signed value, the new high bits are filled with copies of the sign bit. This is called sign extension. For example, sign-extending the 8-bit value $80 (-128) to 16 bits sets the new bits, resulting in $FF80; sign-extending $7F (+127) to 16 bits results in $007F. The following sequences calculate the upper byte of the sign-extended value.

        ; calculates upper byte of sign-extension of A
        ora #$7F
        bmi neg
        lda #0
neg:
        ; calculates upper byte of sign-extension of A, alternate version
        and #$80
        bpl pos
        lda #$ff
pos:

;This constant-time version (7 bytes, 8 cycles) destroys the carry, so don't try using it in the middle of a multi-byte addition:

  asl a     ;  cpx #$80 or cpy #$80 is also possible
  lda #$00
  adc #$FF  ; C is unchanged and A = $00 if C was set or $FF if C was clear
  eor #$FF  ; now all bits of A are set to what bit 7 was

;If you're just trying to add an 8-bit delta to a 16-bit value, you could try subtracting 256 from the value by decrementing the high byte if the value is negative and then adding as if it were unsigned.

  lda delta_value
  bpl notneg
  dec value_hi
notneg:
  clc
  adc value_lo
  sta value_lo
  lda #0
  adc value_hi
  sta value_hi